<?php

// dynamic recount
add_action('wcpt_parse_attributes', 'wcpt_dynamic_recount', 10, 1);
function wcpt_dynamic_recount($attributes){
	$data = wcpt_get_table_data();

	if( 
    empty( $data['query']['sc_attrs'] ) ||
    (
      empty( $data['query']['sc_attrs']['dynamic_recount'] ) && 
      empty( $data['query']['sc_attrs']['dynamic_hide_filters'] )
    )
	){
   	// remove filters applied by prev table
		remove_filter('wcpt_query_args', 'wcpt_global_query_args');
		remove_filter('wcpt_nav_filter_option', 'wcpt_term_count_placeholder');
		remove_filter('wcpt_navigation', 'wcpt_show_term_count');

		return $attributes;
	}

	// add filters
  // -- add count placeholders
  add_filter('wcpt_nav_filter_option', 'wcpt_term_count_placeholder', 100, 3);

  // -- globally store current wcpt instance query args
  add_filter('wcpt_query_args', 'wcpt_global_query_args', 100, 1);

  // -- get each filter option count and replace placeholders
  add_filter('wcpt_navigation', 'wcpt_show_term_count', 100, 1);

	return $attributes;
}

// globally store current wcpt instance query args
function wcpt_global_query_args( $query_args ){
	$GLOBALS['wcpt_query_args'] = $query_args;
  return $query_args;
}

// add count placeholders
function wcpt_term_count_placeholder( $option, $filter, $params= null ){
  // option must have value and label keys

  if( empty( $GLOBALS['wcpt_filter_counts'] ) ){
    $GLOBALS['wcpt_filter_counts'] = array();
  }

  global $wcpt_filter_counts;

  $count_id = 'wcpt-count-' . rand(0, 10000000000);
  $wcpt_filter_counts[] = array(
    'filter' 	 => $filter,
    'value'		 => gettype($option) === 'object' ? $option->value : $option['value'],
    'taxonomy' => ! empty( $params ) && ! empty( $params['taxonomy'] ) ? $params['taxonomy'] : null,
    'id' 			 => $count_id,
  );

  if( gettype($option) === 'array' ){
    $option['label'] .= $count_id;
  }else if( gettype($option) === 'object' ){
    $option->label .= $count_id;
  }

  return $option;
}

// get each filter option count and replace placeholders
function wcpt_show_term_count( $nav_mkp ){
  $data = wcpt_get_table_data();

  if( empty( $data['query']['sc_attrs'] ) ){
    $sc_attrs = array();
  }else{
    $sc_attrs = $data['query']['sc_attrs'];
  }

  $query_args = $GLOBALS['wcpt_query_args'];
  $query_args['posts_per_page'] = -1;
  $query_args['fields'] = 'ids';

  $max_count = 200;

  $attribute_relation__or = false;
  if(
    ! empty( $sc_attrs['attribute_relation'] ) && 
    strtoupper( $sc_attrs['attribute_relation'] ) === 'OR' &&
    ! empty( $query_args['tax_query']['wcpt_attributes__or'] )
  ){
    $attribute_relation__or = true;
  }

  $dynamic_recount = ! empty( $sc_attrs['dynamic_recount'] );
  $dynamic_hide_filters = ! empty( $sc_attrs['dynamic_hide_filters'] );

  // do not need actual count for term
  // knowing 1 is available is enough
  if( ! $dynamic_recount ){
    $max_count = 1;
  }

  // reduce total number of products to search through the db
  $query = new WP_Query( $query_args );
  $post_ids = $query->posts;

  $attribute_taxonomies = array();
  if( ! empty( $query_args['tax_query'] ) ){

    if( $attribute_relation__or ){
      foreach( $query_args['tax_query']['wcpt_attributes__or'] as $arr ){
        if( is_array( $arr ) && ! empty( $arr['taxonomy'] ) ){
          $attribute_taxonomies[] = $arr['taxonomy'];
        }
      }
    }else{
      foreach( $query_args['tax_query'] as $arr ){
        if( is_array( $arr ) && ! empty( $arr['taxonomy'] ) ){
          $attribute_taxonomies[] = $arr['taxonomy'];
        }
      }
    }
  }

  $category_filtering = false;
  if( ! empty( $query_args['tax_query'] ) ){
    foreach( $query_args['tax_query'] as $arr ){
      if(
        is_array( $arr ) && 
        ! empty( $arr['taxonomy'] ) &&
        $arr['taxonomy'] == 'product_cat'
      ){
        $category_filtering = true;
        break;
      }
    }
  }

  $filters =& $GLOBALS['wcpt_filter_counts'];

  if( empty( $filters ) ){
    return $nav_mkp;
  }

  $hide_selectors = array();

  foreach( $filters as &$option ){

    // availability
    if( $option['filter'] == 'attribute' ){

      // if( count( $post_ids ) || $attribute_relation__or ){

        if( $attribute_relation__or ){
          $_query_args = $query_args;
          unset( $_query_args['tax_query']['wcpt_attributes__or'] );
          $_query_args['tax_query'][] = array(
            'taxonomy' => $option['taxonomy'],
            'field'    => 'term_taxonomy_id',
            'terms'    => $option['value'],
          );
          $query = new WP_Query( $_query_args );
          $_count = $query->post_count;

        }else{

          // attribute is not being used by user to filter
          if( ! in_array( $option['taxonomy'], $attribute_taxonomies ) ){

            if( count( $post_ids ) ){
              $query = new WP_Query( array(
                'post_type' => 'product',
                'posts_per_page' => ($max_count + 1),
                'post__in' => $post_ids,
                'fields' => 'ids',
                'tax_query' => array(
                  array(
                    'taxonomy' => $option['taxonomy'],
                    'field'    => 'term_taxonomy_id',
                    'terms'    => $option['value'],
                  ),
                ),
              ) );
              $_count = $query->post_count;

            }else{
              $_count = 0;
            }

          // attribute is being used by user to filter
          }else{

            $_query_args = $query_args;
            $_tax_query = array();

            foreach( $_query_args['tax_query'] as $key => $val ){
              if( $key === 'relation' ){
                $_tax_query['relation'] = $val;

              }else if( $val['taxonomy'] !== $option['taxonomy'] ){
                $_tax_query[] = $val;
              }
            }

            $_tax_query[] = array(
              'taxonomy' => $option['taxonomy'],
              'field'    => 'term_taxonomy_id',
              'terms'    => $option['value'],
            );

            $_query_args['tax_query'] = $_tax_query;
            $query = new WP_Query( $_query_args );
            $_count = $query->post_count;

          }

        }

        // does this filter option have a count?
        $count = $_count > $max_count ? $max_count . '+' : $_count;

      // }

      if( ! $count ){
        $hide_selectors[] = ' [data-wcpt-filter="'. $option['filter'] .'"].wcpt-filter [data-wcpt-value="'. $option['value'] .'"]';
      }

      $nav_mkp = str_replace( $option['id'], ' <span class="wcpt-count" style="color: #999;">('. $count .')</span>', $nav_mkp );

    // category
    }else if( $option['filter'] == 'category' ){

      // if( count( $post_ids ) ){
        // product category is not being used by user to filter
        if( ! $category_filtering ){
          $query = new WP_Query( array(
            'post_type' => 'product',
            'posts_per_page' => ($max_count + 1),
            'post__in' => $post_ids,
            'fields' => 'ids',
            'tax_query' => array(
              array(
                'taxonomy' => $option['taxonomy'],
                'field'    => 'term_taxonomy_id',
                'terms'    => $option['value'],
              ),
            ),
          ) );

        // product category is being used by user to filter
        }else{
          $_query_args = $query_args;
          $_tax_query = array();

          // exclude prpduct categories from new query 
          foreach( $_query_args['tax_query'] as $key => $val ){
            if( gettype( $key ) === 'string' ){
              $_tax_query[ $key ] = $val;

            }else if( $val['taxonomy'] !== $option['taxonomy'] ){
              $_tax_query[] = $val;

            }
          }

          // only this category will be allowed
          $_tax_query[] = array(
            'taxonomy' => $option['taxonomy'],
            'field'    => 'term_taxonomy_id',
            'terms'    => $option['value'],
          );

          $_query_args['tax_query'] = $_tax_query;
          $query = new WP_Query( $_query_args );

        }

        // does this filter option have a count?
        $count = $query->post_count > $max_count ? $max_count . '+' : $query->post_count;

      // }else{
      //   $count = 0;
      // }

      if( ! $count ){
        $hide_selectors[] = ' [data-wcpt-filter="'. $option['filter'] .'"].wcpt-filter [data-wcpt-value="'. $option['value'] .'"]';
      }

      $nav_mkp = str_replace( $option['id'], ' <span class="wcpt-count" style="color: #999;">('. $count .')</span>', $nav_mkp );

    // sale
    } else if( $option['filter'] == 'on_sale' ){

      // does this filter option have a count?
      $count = count( array_intersect( wc_get_product_ids_on_sale(), $post_ids ) );

      if( ! $count ){
        $hide_selectors[] = ' [data-wcpt-filter="'. $option['filter'] .'"].wcpt-filter [data-wcpt-value="'. ($option['value'] ? 'true' : 'false') .'"]';
      }

      $nav_mkp = str_replace( $option['id'], ' <span class="wcpt-count" style="color: #999;">('. $count .')</span>', $nav_mkp );

    // availability
    } else if( $option['filter'] == 'availability' ){

      $product_visibility_terms  = wc_get_product_visibility_term_ids();

      if( count( $post_ids ) ){
        $query = new WP_Query( array(
          'post_type' => 'product',
          'posts_per_page' => ($max_count + 1),
          'post__in' => $post_ids,
          'fields' => 'ids',
          'tax_query' => array(
            array(
              'taxonomy' 	=> 'product_visibility',
              'field'    => 'term_taxonomy_id',
              'terms'			=> array( $product_visibility_terms['outofstock'] ),
              'operator'	=> $option['value'],
            ),
          ),
        ) );

        // does this filter option have a count?
        $count = $query->post_count > $max_count ? $max_count . '+' : $query->post_count;

      }else{
        $count = 0;

      }

      if( ! $count ){
        $hide_selectors[] = ' [data-wcpt-filter="'. $option['filter'] .'"].wcpt-filter [data-wcpt-value="'. $option['value'] .'"]';
      }

      $nav_mkp = str_replace( $option['id'], ' <span class="wcpt-count" style="color: #999;">('. $count .')</span>', $nav_mkp );

    }

    $option['count'] = $count;
  }

  if( $dynamic_hide_filters ){

    // now that each of the filter options has a 'count' assigned
    // hide filters with all options hidden

    $attributes = array();
    $keep_attributes = array();
    $hide_attributes = array();

    $keep_on_sale = false;
    $keep_availability = false;

    foreach( $filters as $_option ){

      if( $_option['filter'] === 'attribute' ){
        // +attributes
        if( ! in_array( $_option['taxonomy'], $attributes ) ){
          $attributes[] = $_option['taxonomy'];
        }
        // +keep attributes
        if(
          $_option['count'] &&
          ! in_array( $_option['taxonomy'], $keep_attributes )
        ){
          $keep_attributes[] = $_option['taxonomy'];
        }

      }else if( 
        $_option['filter'] === 'on_sale' && 
        $_option['count']
      ){
        $keep_on_sale = true;

      }else if(
        $_option['filter'] === 'availability' && 
        $_option['count']
      ){
        $keep_availability = true;

      }

    }

    $hide_attributes = array_diff($attributes, $keep_attributes);
    foreach( $hide_attributes as $taxonomy ){
      $hide_selectors[] = ' .wcpt-filter[data-wcpt-taxonomy="' . $taxonomy . '"] ';
    }

    if( ! $keep_on_sale ){
      $hide_selectors[] = ' .wcpt-filter[data-wcpt-filter="on_sale"] ';
    }

    if( ! $keep_availability ){
      $hide_selectors[] = ' .wcpt-filter[data-wcpt-filter="availability"] ';
    }

    $style = '';
    if( count( $hide_selectors ) ){
      $style = '<style>.wcpt-' . $data['id'] . ' ' . implode( ', .wcpt-' . $data['id'] . ' ', $hide_selectors ) . '{ display: none !important; } </style>';
    }

    $nav_mkp = $style . $nav_mkp;

  }

  // hide counts if not enabled
  if( ! $dynamic_recount ){
    $nav_mkp = '<style>.wcpt-' . $data['id'] . ' .wcpt-count { display: none !important; } </style>' . $nav_mkp;
  }

  $filters = array(); // reset for next table
  return $nav_mkp;

}