<?php
// PRO flag
// -- PHP
add_action( 'init', 'wcpt_pro_flag_php', 100 );
function wcpt_pro_flag_php(){
  if( ! defined( 'WCPT_PRO' ) ){
    define( 'WCPT_PRO', TRUE );
  }
}

// -- JS
add_action( 'admin_print_scripts', 'wcpt_pro_flag_js' );
function wcpt_pro_flag_js(){
  ob_start();
  ?>
  var wcpt_pro = true;
  jQuery(function($){
    $('body').addClass('wcpt-pro');
  })
  <?php
  wp_add_inline_script( 'jquery', ob_get_clean() );
}

// deactivate lite
register_activation_hook( __FILE__, 'wcpt_deactivate_lite' );
add_action( 'admin_init', 'wcpt_deactivate_lite' );
function wcpt_deactivate_lite(){
  $lite = 'wc-product-table-lite/main.php';
  if ( is_plugin_active( $lite ) ) {
    deactivate_plugins( $lite );
    add_action( 'admin_notices', 'wcpt_lite_deactivation_notice' );
  }
}

function wcpt_lite_deactivation_notice() {
	$class = 'notice notice-warning';
	$message = __( 'WCPT Lite has been deactivated by WCPT PRO. If you wish to use WCPT Lite, please de-activate WCPT PRO first. There can only be one.', 'wc-product-table' );
	printf( '<div class="%1$s"><p>%2$s</p></div>', esc_attr( $class ), esc_html( $message ) );
}

// EDD

//-- update PRO
define( 'WCPT_PRO_URL', 'https://pro.wcproducttable.com' );
define( 'WCPT_PRO_ITEM_ID', 10 );

add_action('admin_init', 'wcpt_edd_update_pro');
function wcpt_edd_update_pro(){
  if( ! class_exists( 'EDD_SL_Plugin_Updater' ) ) {
  	include( dirname( __FILE__ ) . '/EDD_SL_Plugin_Updater.php' );
  }
  $edd_updater = new EDD_SL_Plugin_Updater(
    WCPT_PRO_URL,
    WCPT_PLUGIN_PATH . 'main.php',
    array(
  		'version' 	=> WCPT_VERSION, 	// current version number
  		'license' 	=> trim( get_option( 'wcpt_pro_license_key' ) ),
  		'item_id'   => WCPT_PRO_ITEM_ID, 	// id of this product in EDD
  		'author' 	  => 'Kartik Gahlaut',  // author of this plugin
  		'url'       => home_url(),
  	)
  );
}

//-- manage license
add_action( 'wp_ajax_wcpt_manage_license', 'wcpt_manage_license' );
function wcpt_manage_license(){
  if(
    empty( $_POST['wcpt_purpose'] ) ||
    empty( $_POST['wcpt_key'] ) ||
    empty( $_POST['wcpt_nonce'] ) ||
    ! wp_verify_nonce( $_POST['wcpt_nonce'], 'wcpt' )
  ){
    die();
  }

  $url = WCPT_PRO_URL . '/?edd_action='. $_POST['wcpt_purpose'] .'_license&item_id='. WCPT_PRO_ITEM_ID .'&license='. $_POST['wcpt_key'] .'&url='. home_url();
	$response = wp_remote_get( $url, array( 'timeout' => 30, 'sslverify' => true ) );

  if( is_wp_error( $response ) ){
    die();
  }

  $response = json_decode( $response['body'], true );

  if( $response['license'] == 'invalid' ){

    if( $response['error'] == 'no_activations_left' ){
      echo 'active_elsewhere';
      die();
    }

    echo 'invalid_key';
    die();

  }

  if( $response['success'] ){
    if( $_POST['wcpt_purpose'] == 'activate' ){
      wcpt_set_license_status( 'active' );
    }

    if( $_POST['wcpt_purpose'] == 'deactivate' ){
      wcpt_set_license_status( 'inactive' );
    }

    // save license key
    wcpt_update_license_key($_POST['wcpt_key']);
    update_option( 'wcpt_pro_license_key', $_POST['wcpt_key'] );
  }


  echo $_POST['wcpt_purpose'] . 'd'; // activated / deactivated

  die();
}

//-- license status : active / inactive
function wcpt_get_license_status(){
  return get_option('wcpt_license_status');
}

function wcpt_set_license_status($status){
  update_option('wcpt_license_status', $status);
}

function wcpt_update_license_key($key){
  $global_settings = wcpt_get_settings_data();
  if( $global_settings ){
    $global_settings['pro_license']['key'] = $key;
  }
  update_option( 'wcpt_settings', addslashes( json_encode( $global_settings) ) );
}

//-- update license key when user changes it
add_filter('wcpt_global_settings', 'wcpt_license_key_update', 10, 1);
function wcpt_license_key_update($data){
  $_data = json_decode( stripslashes($data), true );
  if( ! empty( $_data['pro_license'] ) && ! empty( $_data['pro_license']['key'] ) ){
    update_option( 'wcpt_pro_license_key', $_data['pro_license']['key'] );
  }
  return $data;
}

// session required
add_action('init', 'wcpt_session');
function wcpt_session(){
  if( 
    ! headers_sent() &&
    ! session_id() 
  ){
    session_start();
  }
}

// plugin activation meta link
add_filter('plugin_row_meta', 'wcpt_plugin_row_meta', 10, 2);
function wcpt_plugin_row_meta( $links, $file ){
  if( WCPT_PRO ){
    $text = wcpt_get_license_status() == 'active' ? esc_html__('Manage license', 'wc-product-table') : esc_html__('Activate license', 'wc-product-table');
    if ( $file == plugin_basename('wc-product-table-pro/main.php') ) {
  		$links[] = '<a href="' . get_admin_url() . 'edit.php?post_type=wc_product_table&page=wcpt-settings#pro_license">' . $text . '</a>';
  	}
  }
	return $links;
}

// archive override
function wcpt_archive_override(){
  // shortcode attributes
  $wcpt_table_id = '';
  $wcpt_custom_table = false;
  $category = '';
  $tag = '';
  $attribute = '';
	$page ='';
	$search = '';
	if( get_query_var('paged') !== 1 ){
		$page = " page='". get_query_var('paged') ."' ";
	}

  // get global settings
	$wcpt_settings = wcpt_get_settings_data();

	if( ! empty( $wcpt_settings['archive_override'] ) ){

		// get global default
		if( empty( $wcpt_settings['archive_override']['default'] ) ){
			$wcpt_settings['archive_override']['default'] = '';
		}

    $wcpt_table_id = $wcpt_settings['archive_override']['default'];
    if( $wcpt_table_id == 'custom' ){
      $wcpt_custom_table = $wcpt_settings['archive_override']['default_custom'];
    }

		// search
		if ( is_search() ) {
      // Product search results page
			global $wp_query;
			$search = " _search='". get_query_var("s") ."' ";

			// shop archive override table
			//-- none
			if( empty( $wcpt_settings['archive_override']['search'] ) ){
				$wcpt_table_id = '';

			//-- default
  		}else if( $wcpt_settings['archive_override']['search'] == 'default' ){
        $wcpt_table_id = $wcpt_settings['archive_override']['default'];
        
			//-- custom
      }else if( $wcpt_settings['archive_override']['search'] == 'custom' ){
        $wcpt_table_id = 'custom';
        $wcpt_custom_table = $wcpt_settings['archive_override']['search_custom'];

			//-- a table is assigned
			}else{
				$wcpt_table_id = $wcpt_settings['archive_override']['search'];

			}

		// shop
		}else if ( is_shop() ) {

			// shop archive override table
			//-- none
			if( empty( $wcpt_settings['archive_override']['shop'] ) ){
				$wcpt_table_id = '';

			//-- default
			}else if( $wcpt_settings['archive_override']['shop'] == 'default' ){
        $wcpt_table_id = $wcpt_settings['archive_override']['default'];
        
			//-- custom
      }else if( $wcpt_settings['archive_override']['shop'] == 'custom' ){
        $wcpt_table_id = 'custom';
        $wcpt_custom_table = $wcpt_settings['archive_override']['shop_custom'];

			//-- a table is assigned
			}else{
				$wcpt_table_id = $wcpt_settings['archive_override']['shop'];

			}

			$category = '';
		  $tag = '';
		  $attribute = '';

		// category
		}else if ( is_product_category() ){
			$_category = get_query_var("product_cat", "");
	    $category = " category='". esc_attr( $_category ) ."' ";

			// default category archive override table
			//-- none
			if( empty( $wcpt_settings['archive_override']['category']['default'] ) ){
				$wcpt_table_id = '';

			//-- default
			}else if( $wcpt_settings['archive_override']['category']['default'] == 'default' ){
				$wcpt_table_id = $wcpt_settings['archive_override']['default'];

			//-- custom
      }else if( $wcpt_settings['archive_override']['category']['default'] == 'custom' ){
        $wcpt_table_id = 'custom';
        $wcpt_custom_table = $wcpt_settings['archive_override']['category']['custom'];

			//-- a category default table is assigned
			}else{
				$wcpt_table_id = $wcpt_settings['archive_override']['category']['default'];

			}

			// look through other category rules
			if( ! empty( $wcpt_settings['archive_override']['category']['other_rules'] ) ){
				foreach( $wcpt_settings['archive_override']['category']['other_rules'] as $rule ){

					// apply rule
					if( ! empty( $rule['category'] ) && in_array( $_category, $rule['category'] ) ){
						$wcpt_table_id = $rule['table_id'];
						// use default
						if( $wcpt_table_id == 'default' ){
              $wcpt_table_id = $wcpt_settings['archive_override']['category']['default'];
              
						}else if( $wcpt_table_id == 'custom' ){
              $wcpt_custom_table = $rule['custom'];

            }
            
					}

				}
			}

		  $tag = '';
      $attribute = '';
      
      // handling subcategories grid
      if( ! defined( 'WCPT_FLAG_CAT_ARCHIVE' ) ){
        switch ( woocommerce_get_loop_display_mode() ) {
          case 'subcategories':
            return false;
  
          case 'both':
            add_action( 'woocommerce_after_shop_loop', 'wcpt_archive_override' );
            define('WCPT_FLAG_CAT_ARCHIVE', TRUE);
            return false;
        }
      }else{
        wp_reset_query();

        if( $wcpt_table_id ){
          echo '<style> #main .products .product:not(.product-category){ display: none !important; } </style>';
        }
      }

		}else if( is_product_tag() ) {
			$_tag = get_query_var('term', '');
			$tag = " taxonomy='product_tag:". $_tag ."' ";

			// default tag archive override table
			//-- none
			if( empty( $wcpt_settings['archive_override']['tag']['default'] ) ){
				$wcpt_table_id = '';

			//-- default
			}else if( $wcpt_settings['archive_override']['tag']['default'] == 'default' ){
				$wcpt_table_id = $wcpt_settings['archive_override']['default'];

			//-- custom
      }else if( $wcpt_settings['archive_override']['tag']['default'] == 'custom' ){
        $wcpt_table_id = 'custom';
        $wcpt_custom_table = $wcpt_settings['archive_override']['tag']['custom'];

			//-- a tag default table is assigned
			}else{
				$wcpt_table_id = $wcpt_settings['archive_override']['tag']['default'];

			}

			// look through other tag rules
			if( ! empty( $wcpt_settings['archive_override']['tag']['other_rules'] ) ){
				foreach( $wcpt_settings['archive_override']['tag']['other_rules'] as $rule ){

					// apply rule
					if( ! empty( $rule['tag'] ) && in_array( $_tag, $rule['tag'] ) ){
						$wcpt_table_id = $rule['table_id'];
						// use default
						if( $wcpt_table_id == 'default' ){
              $wcpt_table_id = $wcpt_settings['archive_override']['tag']['default'];
              
						}else if( $wcpt_table_id == 'custom' ){
              $wcpt_custom_table = $rule['custom'];

            }
					}

				}
			}

			$category = '';
		  $attribute = '';

	  }else if( is_product_taxonomy() ){

			$taxonomy = get_query_var('taxonomy', false);
			$_attribute = substr( $taxonomy, 3 );

      $attribute = " attribute='". wc_attribute_label( $taxonomy ) .":". get_query_var('term', '') ."' ";

			// default attribute archive override table
			//-- none
			if( empty( $wcpt_settings['archive_override']['attribute']['default'] ) ){
				$wcpt_table_id = '';

			//-- default
			}else if( $wcpt_settings['archive_override']['attribute']['default'] == 'default' ){
        $wcpt_table_id = $wcpt_settings['archive_override']['default'];
        
			//-- custom
      }else if( $wcpt_settings['archive_override']['attribute']['default'] == 'custom' ){
        $wcpt_table_id = 'custom';
        $wcpt_custom_table = $wcpt_settings['archive_override']['attribute']['custom'];

			//-- a attribute default table is assigned
			}else{
				$wcpt_table_id = $wcpt_settings['archive_override']['attribute']['default'];

			}

			// look through other attribute rules
			if( ! empty( $wcpt_settings['archive_override']['attribute']['other_rules'] ) ){
				foreach( $wcpt_settings['archive_override']['attribute']['other_rules'] as $rule ){

					// apply rule
					if( ! empty( $rule['attribute'] ) && in_array( $_attribute, $rule['attribute'] ) ){
						$wcpt_table_id = $rule['table_id'];
						// use default
						if( $wcpt_table_id == 'default' ){
							$wcpt_table_id = $wcpt_settings['archive_override']['attribute']['default'];
						}
					}

				}
			}

			$category = '';
		  $tag = '';

		}

	}

  if( $wcpt_table_id ){

    // hide the table navigation and depend
    // upon external filter to direct query
    // in these cases:
		$only_loop = '';
		if( ! function_exists( 'is_plugin_active' ) ){
			include_once( ABSPATH . 'wp-admin/includes/plugin.php' );
		}
    $theme = wp_get_theme();
    if(
			is_plugin_active( 'prdctfltr/prdctfltr.php' ) ||
			is_plugin_active( 'woocommerce-products-filter/index.php' ) ||
			is_plugin_active( 'woocommerce-ajax-layered-nav/ajax_layered_nav-widget.php' ) ||
      ( 'Woodmart' == $theme->name || 'Woodmart' == $theme->parent_theme )
		){
			$only_loop = " _only_loop='true'  ";
    }
    
    if( $wcpt_custom_table ){
      // replace force required shortcode attributes into the custom shortcode
      $shortcode = str_replace("]", " _archive='true' {$category} {$attribute} {$tag} {$page} {$search} {$only_loop} ]", $wcpt_custom_table);
    }else{
      $shortcode = "[product_table id='". $wcpt_table_id ."' _archive='true' {$category} {$attribute} {$tag} {$page} {$search} {$only_loop} ]";
    }

    echo do_shortcode( apply_filters('wcpt_archive_override_filter', $shortcode) );

		if( ! $only_loop ){
      // general - hide result count and orderby
  		echo '<style media="screen">.woocommerce-result-count,.woocommerce-ordering,.woocommerce-pagination {display: none !important;}</style>';
      // martfury - hide result count and orderby
  		echo '<style media="screen">#mf-catalog-toolbar, #mf-catalog-toolbar + .mf-toolbar-empty-space {display: none;}</style>';
		}

    return true;

	}

  return false;
}

// json-ld
//-- generate
add_filter( 'wcpt_product', 'wcpt_generate_json_ld', 10, 2 );
function wcpt_generate_json_ld( $product, $attributes ){
  if(
    ! wp_doing_ajax() &&
    ! empty( $attributes['_archive'] ) ||
    (
      ! empty( $attributes['sc_attrs'] ) &&
      ! empty( $attributes['sc_attrs']['json_ld'] )
    )
  ){
    if( $GLOBALS['wcpt_device'] == 'laptop' ){ // don't want duplicates
      WC()->structured_data->generate_product_data($product);
    }
  }

  return $product;
}

//-- decision: print
add_action( 'wcpt_before_loop', 'wcpt_maybe_print_json_ld', 10 );
function wcpt_maybe_print_json_ld( $attributes ){
  if(
    // json-ld requested
    ! empty( $attributes['sc_attrs'] ) &&
    ! empty( $attributes['sc_attrs']['json_ld'] ) &&
    // not a page where WC prints it
    ! is_woocommerce()
  ){
		add_action( 'wp_footer', 'wcpt_print_json_ld', 10 );
  }
}

//-- print
function wcpt_print_json_ld(){
  if ( $data  = WC()->structured_data->get_structured_data( array('product') ) ){
    echo '<script type="application/ld+json">' . wp_json_encode( $data ) . '</script>';
  }
}

// duplicate post
add_action( 'admin_action_wcpt_duplicate_post_as_draft', 'wcpt_duplicate_post_as_draft' );
function wcpt_duplicate_post_as_draft(){
	global $wpdb;
	if( ! ( isset( $_GET['post']) || isset( $_POST['post'])  || ( isset($_REQUEST['action']) && 'wcpt_duplicate_post_as_draft' == $_REQUEST['action'] ) ) ){
		wp_die('No post to duplicate has been supplied!');
	}

	/*
	 * Nonce verification
	 */
	if (
     ! isset( $_GET['duplicate_nonce'] ) ||
     ! wp_verify_nonce( $_GET['duplicate_nonce'], WCPT_PLUGIN_PATH )
   ){
		wp_die('Busted!');
  }

	/*
	 * get the original post id
	 */
	$post_id = ( isset( $_GET['post'] ) ? absint( $_GET['post'] ) : absint( $_POST['post'] ) );

	/*
	 * and all the original post data then
	 */
	$post = get_post( $post_id );

	/*
	 * if you don't want current user to be the new post author,
	 * then change next couple of lines to this: $new_post_author = $post->post_author;
	 */
	$current_user = wp_get_current_user();
	$new_post_author = $current_user->ID;

	/*
	 * if post data exists, create the post duplicate
	 */
	if( isset( $post ) && $post != null ){

		/*
		 * new post data array
		 */
		$args = array(
			'comment_status' => $post->comment_status,
			'ping_status'    => $post->ping_status,
			'post_author'    => $new_post_author,
			'post_content'   => $post->post_content,
			'post_excerpt'   => $post->post_excerpt,
			'post_name'      => $post->post_name,
			'post_parent'    => $post->post_parent,
			'post_password'  => $post->post_password,
			'post_status'    => 'publish',
			'post_title'     => $post->post_title,
			'post_type'      => $post->post_type,
			'to_ping'        => $post->to_ping,
			'menu_order'     => $post->menu_order
		);

		/*
		 * insert the post by wp_insert_post() function
		 */
		$new_post_id = wp_insert_post( $args );

		/*
		 * get all current post terms ad set them to the new post draft
		 */
		$taxonomies = get_object_taxonomies($post->post_type); // returns array of taxonomy names for post type, ex array("category", "post_tag");
		foreach ($taxonomies as $taxonomy) {
			$post_terms = wp_get_object_terms($post_id, $taxonomy, array('fields' => 'slugs'));
			wp_set_object_terms($new_post_id, $post_terms, $taxonomy, false);
		}

		/*
		 * duplicate all post meta just in two SQL queries
		 */
		$post_meta_infos = $wpdb->get_results("SELECT meta_key, meta_value FROM $wpdb->postmeta WHERE post_id=$post_id");
		if (count($post_meta_infos)!=0) {
			$sql_query = "INSERT INTO $wpdb->postmeta (post_id, meta_key, meta_value) ";
			foreach ($post_meta_infos as $meta_info) {
				$meta_key = $meta_info->meta_key;
				if( $meta_key == '_wp_old_slug' ) continue;
				$meta_value = addslashes($meta_info->meta_value);
				$sql_query_sel[]= "SELECT $new_post_id, '$meta_key', '$meta_value'";
			}
			$sql_query.= implode(" UNION ALL ", $sql_query_sel);
			$wpdb->query($sql_query);
		}

    // provide new ids to avoid conflicts
    if( $table_data = get_post_meta( $post_id, 'wcpt_data', true ) ){
      $table_data = json_decode($table_data, true);
      wcpt_new_ids( $table_data ); // recursively iterates and switches ids to new randoms
      $table_data['id'] = $new_post_id; // this is the table id and should not be random
      update_post_meta( $new_post_id, 'wcpt_data', addslashes(json_encode( $table_data )) );
    }

		/*
		 * finally, redirect to the edit post screen for the new draft
		 */
		wp_redirect( admin_url( 'post.php?action=edit&post=' . $new_post_id ) );
		exit;
	} else {
		wp_die('Post creation failed, could not find original post: ' . $post_id);
	}
}

function wcpt_new_ids( &$arr, $fresh= true ){
  if( $fresh ){
    $GLOBALS['wcpt_new_ids_id'] = time() + ( rand(0, 100000) );
  }
  global $wcpt_new_ids_id;
  foreach( $arr as $key => &$val ){
    if( $key === 'id' ){
      $val = ++$wcpt_new_ids_id;
    }else if( gettype($val) == 'array' ){
      wcpt_new_ids( $val, false );
    }
  }
}

add_action( 'wcpt_before_apply_user_filters', 'wcpt_pro_shortcode_attributes', 100, 1 );
function wcpt_pro_shortcode_attributes($data){
	$sc_attrs = $data['query']['sc_attrs'];

  //-- attributes
  if( ! empty( $sc_attrs['attribute'] ) ){
    $attrs = explode('|', $sc_attrs['attribute']);
    foreach( $attrs as $attribute_string ){
      $split = explode(':', trim( $attribute_string ));
      $taxonomy = $split[0];
      if( ! $split[0] ){
        continue;
      }
      $slug = sanitize_title($taxonomy);
      // already a slug
      if( $taxonomy == $slug ){
        // ensure pa_
        if( 'pa_' !== substr( $slug, 0, 3 ) ){
          $taxonomy = 'pa_' . $slug;
        }

      }else{
        // get slug from name
        $id = wc_attribute_taxonomy_id_by_name($taxonomy);
        $attribute = wc_get_attribute($id);
        $taxonomy = $attribute->slug;

      }
      if( ! empty( $split[1] ) ){
        foreach( explode(',', $split[1]) as $term ){
          $term = get_term_by( 'slug', trim($term), $taxonomy );
          if( $term && ! is_wp_error( $term ) ){
            $filter_info = array(
              'filter'      => 'attribute',
              'values'      => array( $term->term_taxonomy_id ),
              'taxonomy'    => $taxonomy,
              'operator'    => 'IN',
              'clear_label' => '',
              'clear_labels_2' => '',
            );
            wcpt_update_user_filters($filter_info, false);
          }
        }
      }
    }
  }
  //-- taxonomy
  if( ! empty( $sc_attrs['taxonomy'] ) ){
    $taxs = explode('|', $sc_attrs['taxonomy']);
    foreach( $taxs as $taxonomy_string ){
      $split = explode(':', trim( $taxonomy_string ));
      $taxonomy = $split[0];
      if( ! $split[0] ){
        continue;
      }
      $taxonomy = sanitize_title($taxonomy);
      $tax_obj = get_taxonomy( $taxonomy );
      if( empty( $tax_obj->public ) || empty( $tax_obj->publicly_queryable ) ){
        continue;
      }
      if( ! empty( $split[1] ) ){
        foreach( explode(',', $split[1]) as $term ){
          $term = get_term_by( 'slug', trim($term), $taxonomy );
          if( $term && ! is_wp_error( $term ) ){
            $filter_info = array(
              'filter'      => 'taxonomy',
              'values'      => array( $term->term_taxonomy_id ),
              'taxonomy'    => $taxonomy,
              'operator'    => 'IN',
              'clear_label' => '',
              'clear_labels_2' => '',
            );
            wcpt_update_user_filters($filter_info, false);
          }
        }
      }
    }
  }
  //-- custom fields
  if( ! empty( $sc_attrs['custom_field'] ) ){
    $cfs = explode('|', $sc_attrs['custom_field']);
    foreach( $cfs as $cf_string ){
      $split = explode(':', trim( $cf_string ));
      $meta_key = trim($split[0]);
      if( ! empty( $split[1] ) ){
        $filter_info = array(
          'filter'      => 'custom_field',
          'meta_key'    => $meta_key,
          'values'      => array_map( 'trim', explode(',', $split[1]) ),
          'compare'    => 'IN',
          'clear_label' => '',
          'clear_labels_2' => '',
        );

        wcpt_update_user_filters($filter_info, false);
      }
    }
  }

  //-- min-max price
  if( ! empty( $sc_attrs['min_price'] ) || ! empty( $sc_attrs['max_price'] ) ){
    $min_price = ! empty( $sc_attrs[ 'min_price' ] ) ? $sc_attrs[ 'min_price' ] : '';
    $max_price = ! empty( $sc_attrs[ 'max_price' ] ) ? $sc_attrs[ 'max_price' ] : '';

    $value = ( $min_price ? wcpt_price( $min_price ) : 0 ) . ( $max_price ? ' - ' . wcpt_price( $max_price ) : '+' );

    // use filter in query
    $filter_info = array(
      'filter'        => 'price_range',
      'values'        => array( $value ),
      'min_price'     => $min_price,
      'max_price'     => $max_price,
      'clear_label'   => 'Price range',
    );

    wcpt_update_user_filters($filter_info, false);
  }
}

// product variation

// -- modify sc_attrs to include id 
add_filter( 'wcpt_shortcode_attributes', 'wcpt_product_variations_auto_id', 100, 1 );
function wcpt_product_variations_auto_id($atts) {
  if( 
    empty( $atts['ids'] ) && 
    ! empty( $GLOBALS['product'] ) &&
    ! is_archive()
  ){
    $atts['ids'] = $GLOBALS['product']->get_id();
  }

  return $atts;
}

// -- special query for varaitions 
function wcpt_product_variations_query($query_args){

  // run a query to get products
  $variable_products = new WP_Query(
    array_merge(
      $query_args,
      array(
        'product_type'    => 'variable',
        'fields'          => 'ids',
        'posts_per_page'  => -1,
      )
    ) 
  );

  $variable_product_ids = $variable_products->posts;

  // run another query to get the variations
  $vp_query_args = array(
    'post_parent__in' => $variable_product_ids,
    'post_type'       => 'product_variation',
    'post_status'     => 'publish',
    'posts_per_page'  => ! empty( $query_args['posts_per_page'] ) ? $query_args['posts_per_page'] : 10,
    'paged'  					=> ! empty( $query_args['paged'] ) ? $query_args['paged'] : 1,
    'tax_query'				=> array(
      'relation' => ( ! empty( $query_args['tax_query'] ) && ! empty( $query_args['tax_query']['relation'] ) ) ? $query_args['tax_query']['relation'] : 'AND',
    ),
    'meta_query'				=> array(
      'relation' => 'AND',
    ),
  );

  // absorb query args
  //-- orderby
  foreach( array( 'meta_key', 'order', 'orderby' ) as $key ){
    if( ! empty( $query_args[$key] ) ){
      $vp_query_args[ $key ] = $query_args[$key];
    }
  }

  if( ! empty( $query_args['tax_query'] ) ){
    // ensure meta query
    foreach( $query_args['tax_query'] as $key=> $tax_query ){
      if( ! is_array( $tax_query ) ){
        continue; // AND / OR
      }

      //-- attributes
      if( 'pa_' == strtolower( substr( $tax_query['taxonomy'], 0, 3 ) ) ){
        $arr = array(
          'key'     => 'attribute_' . $tax_query['taxonomy'],
          'value'   => array(),
          'compare' => 'IN'
        );

        // convert term_taxonomy_id to slugs
        foreach( $tax_query['terms'] as $tt_id ){
          $term = get_term_by( 'term_taxonomy_id', $tt_id, $tax_query['taxonomy'] );
          $arr['value'][] = $term->slug;
        }

        $vp_query_args['meta_query'][] = $arr;
      };

      //-- availability
      if(
        'product_visibility' == $tax_query['taxonomy'] &&
        'NOT IN' == $tax_query['operator']
      ){
        $term_obj = get_term_by( 'id', $tax_query['terms'][0], 'product_visibility' );
        if( $term_obj && ! is_wp_error( $term_obj ) && $term_obj->name == 'outofstock' ){
          $arr = array(
            'key'     => '_stock_status',
            'value'   => 'instock',
            'compare' => '=',
          );
          $vp_query_args['meta_query'][] = $arr;
        }
      };
    }
  }

  //-- on sale
  $on_sale_filter = wcpt_get_nav_filter('on_sale');
  if( $on_sale_filter && $on_sale_filter['value'] ){
    $arr = array(
      'key'     => '_sale_price',
      'value'		=> 0,
      'compare' => '>',
    );
    $vp_query_args['meta_query'][] = $arr;
  };

  // price range
  if( ! empty( $query_args['meta_query'] ) && ! empty( $query_args['meta_query']['price_filter'] ) ){
    $vp_query_args['meta_query']['price_filter'] = $query_args['meta_query']['price_filter'];
  }

  return new WP_Query( $vp_query_args );

}

add_filter( 'wcpt_permitted_shortcode_attributes', 'wcpt_permit_pro_shortcode_attributes', 100, 1 );
function wcpt_permit_pro_shortcode_attributes($permitted){
  return array_merge($permitted, array(
    'cache',
    'category_relation',
    'nav_category',
    'product_variations',
    'include_hidden',
    'attribute',
    'attribute_relation',
    'custom_field',
    'taxonomy',
    'min_price',
    'max_price',
    'dynamic_recount',
    'dynamic_hide_filters',
    'disable_ajax',
    'lazy_load',
    'json_ld',
    'search_orderby',

    'show_related_products',
    'show_upsells',
    'show_cross_sells',

    'sticky_nav_sidebar',
    'sticky_nav_header',

    'category_required',
    'category_required_message',

    'laptop_freeze_left',
    'laptop_freeze_right',

    'tablet_freeze_left',
    'tablet_freeze_right',

    'phone_freeze_left',
    'phone_freeze_right',

    '_archive',
    '_only_loop',
    '_search',
  ));
}

// sticky nav sidebar
// -- style attribute
add_filter('wcpt_nav_sidebar_style', 'wcpt_sticky_nav_sidebar_style_attribute', 10, 1);
function wcpt_sticky_nav_sidebar_style_attribute($style){
  $data = wcpt_get_table_data();
  if( 
    ! empty( $data['query']['sc_attrs'] ) &&
    ! empty( $data['query']['sc_attrs']['sticky_nav_sidebar'] )
  ){
    $top_offset = empty( $data['query']['sc_attrs']['laptop_scroll_offset'] ) ? '0' : ( (int) $data['query']['sc_attrs']['laptop_scroll_offset'] ). 'px';
    $style .= 'top: '. $top_offset;
  }

  return $style;
}

// -- class attribute
add_filter('wcpt_nav_sidebar_class', 'wcpt_sticky_nav_sidebar_class_attribute', 10, 1);
function wcpt_sticky_nav_sidebar_class_attribute($classes){
  $data = wcpt_get_table_data();
  if( 
    ! empty( $data['query']['sc_attrs'] ) &&
    ! empty( $data['query']['sc_attrs']['sticky_nav_sidebar'] )
  ){
    $classes .= ' wcpt-sticky ';
  }

  return $classes;
}

// sticky nav header
// -- style attribute
add_filter('wcpt_nav_header_style', 'wcpt_sticky_nav_header_style_attribute', 10, 1);
function wcpt_sticky_nav_header_style_attribute($style){
  $data = wcpt_get_table_data();
  if( 
    ! empty( $data['query']['sc_attrs'] ) &&
    ! empty( $data['query']['sc_attrs']['sticky_nav_header'] )
  ){
    $top_offset = empty( $data['query']['sc_attrs']['laptop_scroll_offset'] ) ? '0' : ( (int) $data['query']['sc_attrs']['laptop_scroll_offset'] ). 'px';
    $style .= 'top: '. $top_offset;
  }

  return $style;
}

// -- class attribute
add_filter('wcpt_nav_header_class', 'wcpt_sticky_nav_header_class_attribute', 10, 1);
function wcpt_sticky_nav_header_class_attribute($classes){
  $data = wcpt_get_table_data();
  if( 
    ! empty( $data['query']['sc_attrs'] ) &&
    ! empty( $data['query']['sc_attrs']['sticky_nav_header'] )
  ){
    $classes .= ' wcpt-sticky ';
  }

  return $classes;
}

// up sell products
add_filter( 'wcpt_query_args', 'wcpt_upsell_products' );
function wcpt_upsell_products( $query_args ){
  unset( $query_args['p'] );

  $data = wcpt_get_table_data();
  if( 
    ! empty( $data['query']['sc_attrs'] ) &&
    ! empty( $data['query']['sc_attrs']['show_upsells'] ) &&
    ! empty( $GLOBALS['post'] ) &&
    $product = wc_get_product( is_numeric( $data['query']['sc_attrs']['show_upsells'] ) ? $data['query']['sc_attrs']['show_upsells'] : get_the_id() ) 
  ){
    $upsell_ids = $product->get_upsell_ids();

    if( ! isset( $query_args['post__in'] ) ){
      $query_args['post__in'] = array();
    }

    $query_args['post__in'] = array_merge( $query_args['post__in'], $upsell_ids );
  }

  return $query_args;
}

// cross sell products
add_filter( 'wcpt_query_args', 'wcpt_cross_sell_products' );
function wcpt_cross_sell_products( $query_args ){
  unset( $query_args['p'] );

  $data = wcpt_get_table_data();
  if( 
    ! empty( $data['query']['sc_attrs'] ) &&
    ! empty( $data['query']['sc_attrs']['show_cross_sells'] ) &&
    ! empty( $GLOBALS['post'] ) &&
    $product = wc_get_product( is_numeric( $data['query']['sc_attrs']['show_cross_sells'] ) ? $data['query']['sc_attrs']['show_cross_sells'] : get_the_id() )  
  ){
    $cross_sell_ids = $product->get_cross_sell_ids();

    if( ! isset( $query_args['post__in'] ) ){
      $query_args['post__in'] = array();
    }

    $query_args['post__in'] = array_merge( $query_args['post__in'], $cross_sell_ids );
  }

  return $query_args;
}

// related products
add_filter( 'wcpt_query_args', 'wcpt_related_products' );
function wcpt_related_products( $query_args ){
  unset( $query_args['p'] );

  $data = wcpt_get_table_data();
  if( 
    ! empty( $data['query']['sc_attrs'] ) &&
    ! empty( $data['query']['sc_attrs']['show_related_products'] ) &&
    ! empty( $GLOBALS['post'] ) &&
    $product = wc_get_product( is_numeric( $data['query']['sc_attrs']['show_related_products'] ) ? $data['query']['sc_attrs']['show_related_products'] : get_the_id() ) 
  ){
    $related_product_ids = wc_get_related_products($product->get_id(), -1);

    if( ! isset( $query_args['post__in'] ) ){
      $query_args['post__in'] = array();
    }

    $query_args['post__in'] = array_merge( $query_args['post__in'], $related_product_ids );
  }

  return $query_args;
}

// category relation
add_filter('wcpt_query_args', 'wcpt_pro_category_relation', 20, 1);
function wcpt_pro_category_relation( $query_args = array() ) {
  $data = wcpt_get_table_data();
  $original_categories = $data['query']['category'];
  $sc_attrs = $data['query']['sc_attrs'];

  if(
    ! empty( $sc_attrs ) && 
    ! empty( $sc_attrs['category_relation'] ) &&
    strtoupper( $sc_attrs['category_relation'] ) === 'AND'
  ){
    foreach( $query_args['tax_query'] as &$arr ){
      if(
        $arr['taxonomy'] === 'product_cat' && 
        (
          explode( ',', $data['query']['category'] ) != $arr['terms'] ||
          ! empty( $_GET[ $data['id'] . '_product_cat' ] )
        )
      ){
        $arr['operator'] = 'AND';
      }
    }
  }

  return $query_args;
}

add_filter('wcpt_product', 'wcpt_pro_global_post', 100, 1);
function wcpt_pro_global_post($product){
	$GLOBALS['post'] = get_post($product->get_id()); // used by product addons
  return $product;
}

function wcpt_woocommerce_content() {
  if ( is_singular( 'product' ) ) {
      while ( have_posts() ) :
          the_post();
          wc_get_template_part( 'content', 'single-product' );
      endwhile;

  } else {
      ?>

      <?php if ( apply_filters( 'woocommerce_show_page_title', true ) ) : ?>
          <h1 class="page-title"><?php woocommerce_page_title(); ?></h1>
      <?php endif; ?>

      <?php do_action( 'woocommerce_archive_description' ); ?>

      <?php if ( have_posts() ) : ?>

        <?php do_action( 'woocommerce_before_shop_loop' ); ?>

        <?php
        if( ! function_exists('wcpt_archive_override') || ! wcpt_archive_override() ){
          woocommerce_product_loop_start();
          if ( wc_get_loop_prop( 'total' ) ) {
            while ( have_posts() ) {
              the_post();
              do_action( 'woocommerce_shop_loop' );
              wc_get_template_part( 'content', 'product' );
            }
          }
          woocommerce_product_loop_end();
        }
        ?>

        <?php do_action( 'woocommerce_after_shop_loop' ); ?>

      <?php else : ?>

        <?php do_action( 'woocommerce_no_products_found' ); ?>

      <?php
      endif;

  }
}

// attribute relation
add_filter('wcpt_query_args', 'wcpt_pro_attribute_relation', 20, 1);
function wcpt_pro_attribute_relation( $query_args = array() ) {
  $data = wcpt_get_table_data();
  $sc_attrs = $data['query']['sc_attrs'];

  if( 
    ! empty( $sc_attrs ) && 
    ! empty( $sc_attrs['attribute_relation'] ) &&
    in_array( strtoupper( $sc_attrs['attribute_relation'] ), array('OR', 'AND') )
  ){
    $tax_query = array();
    $tax_query_attributes = array(
      'relation' => strtoupper( $sc_attrs['attribute_relation'] ),
    );

    foreach( $query_args['tax_query'] as $arr ){
      $is_attribute = ! empty( $arr['taxonomy'] ) && 'pa_' == substr( $arr['taxonomy'], 0, 3 );

      if( $is_attribute ){
        $tax_query_attributes[] = $arr;

      }else{
        $tax_query[] = $arr;

      }
    }

    $query_args['tax_query'] = $tax_query;
    $query_args['tax_query']['wcpt_attributes__or'] = $tax_query_attributes;
  }

  return $query_args;
}

/* Include modules */

$wcpt_modules = array(
  WCPT_PLUGIN_PATH . 'pro/dynamic_count.php',
  WCPT_PLUGIN_PATH . 'pro/import_export.php',
);

foreach( $wcpt_modules as $module ){
  if( file_exists( $module ) ){
    require_once( $module );
  }  
}

// custom field auto generate values
// ... based on pre-selected filters and not visitor filters

add_action('wcpt_before_apply_user_filters', 'wcpt_store_query_args_before_user_filters', 1000);
function wcpt_store_query_args_before_user_filters(){
  $original = $GLOBALS['wcpt_table_instance']->query_args;
  $GLOBALS['wcpt_query_args_before_user_filters'] = $GLOBALS['wcpt_table_instance']->parse_query_args();
  $GLOBALS['wcpt_table_instance']->query_args = $original;

  WC()->query->remove_ordering_args();
}

function wcpt_auto_generate_custom_field_options($field_name, $type, $order){
  $data = wcpt_get_table_data();

  if( 
    ! current_user_can('edit_wc_product_table', $data['id']) &&
    $arr = wcpt_get_custom_field_options_from_cache() 
  ){
    return $arr;
  }

  if( ! in_array( $order, array( 'ASC', 'DESC' ) ) ){
    $order = 'ASC';
  }

  if( ! in_array( $type, array( 'CHAR', 'NUMERIC' ) ) ){
    $type = 'CHAR';
  }

  $data = wcpt_get_table_data();
  $args = array_merge(
      $GLOBALS['wcpt_query_args_before_user_filters'],
      array( 
        'posts_per_page' => -1, 
        'paged' => 0
      )
    );
  $results = new WP_Query($args);
  $posts = $results->posts;

  if( ! count( $posts ) ){
    return array();
  }

  $orderby_clause = "ORDER BY meta_value $order";
  if( $type == 'NUMERIC' ){
    $orderby_clause = "ORDER BY (meta_value+0) $order";
  }

  global $wpdb;
  $meta_values = $wpdb->get_col( $wpdb->prepare(
    "
      SELECT DISTINCT meta_value
      FROM {$wpdb->prefix}postmeta 
      WHERE meta_key= %s
      AND post_id IN (". implode( ',', $posts ) .")
      $orderby_clause
    ",
    $field_name
  ) );
  $arr = array();
  foreach( $meta_values as $meta_value ){
    $arr[] = array(
      'label' => $meta_value,
      'value' => $meta_value
    );
  }

  wcpt_store_custom_field_options_in_cache($arr, $field_name);

  return $arr;
}

function wcpt_get_custom_field_options_from_cache(){
  return false; // TODO: WPML and other 'switch' compatible

  $data = wcpt_get_table_data();
  $session_cache_key = 'wcpt_table_' . $data['id'] . '_custom_fields';

  if( ! empty( $_SESSION[$session_cache_key] ) ){
    return $_SESSION[$session_cache_key];
  }else{
    return false;
  }
}

function wcpt_store_custom_field_options_in_cache($arr, $field_name){
  $data = wcpt_get_table_data();  
  $session_cache_key = 'wcpt_table_' . $data['id'] . '_custom_fields';
  if( empty( $_SESSION[$session_cache_key] ) ){
    $_SESSION[$session_cache_key] = array();
  }

  $_SESSION[$session_cache_key][$field_name] = $arr;
}

/* PRO shortcodes */

// audio player
add_shortcode( 'wcpt_player', 'wcpt_player' );
function wcpt_player($atts){
	$atts = shortcode_atts( array(
		'src' => ''
	), $atts, 'wcpt_player' );

  global $post;
  $src = get_post_meta( $post->ID, $atts['src'], true );

  if( is_numeric( $src )  ){
    $src = wp_get_attachment_url( $src );
  }

  ob_start();
  ?>
	<div class="wcpt-player" data-wcpt-src="<?php echo $src; ?>">
		<div class="wcpt-player__play-button wcpt-player__button">
			<svg width="24" height="24" viewBox="0 0 24 24" fill="none" stroke="currentColor" stroke-width="2" stroke-linecap="round" stroke-linejoin="round" class="feather feather-play"><polygon points="5 3 19 12 5 21 5 3"/></svg>
		</div>
		<div class="wcpt-player__pause-button wcpt-player__button">
			<svg width="24" height="24" viewBox="0 0 24 24" fill="none" stroke="currentColor" stroke-width="2" stroke-linecap="round" stroke-linejoin="round" class="feather feather-pause"><rect x="6" y="4" width="4" height="16"/><rect x="14" y="4" width="4" height="16"/></svg>
		</div>		
	</div>
  <?php
  return ob_get_clean();
}

// remove from cart
add_shortcode('wcpt_remove', 'wcpt_remove');
function wcpt_remove( $atts ){
	$atts = shortcode_atts( array(
		'style' => ''
  ), $atts, 'wcpt_remove' );
  $html_class = 'wcpt-remove';
  global $product;
  $in_cart = false;
  if( 
    WC() &&
    WC()->cart &&
    WC()->cart->cart_contents 
  ){
    foreach( WC()->cart->cart_contents as $key => $item ){
      if( 
        $item['product_id'] == $product->get_id() ||
        (
          $product->get_type() == 'variation' &&
          $item['variation_id'] == $product->get_id()
        )
      ){
        $in_cart = true;
      }
    }
  }
  if( ! $in_cart ){
    $html_class .= ' wcpt-disabled';
  }
  ob_start();
  wcpt_icon( 'x', $html_class, $atts['style'], null, __('Remove') );
  return ob_get_clean();
}

add_action( 'wp_ajax_wcpt_remove_product', 'wcpt_remove_product' );
add_action( 'wp_ajax_nopriv_wcpt_remove_product', 'wcpt_remove_product' );
function wcpt_remove_product( ){
  $cart = WC()->instance()->cart;
  
  $remove = array();
  foreach( $cart->get_cart_contents() as $key=> $item ){
    if( $item['product_id'] == $_POST['product_id'] ){
      if( 
        ! empty( $_POST['variation_id'] ) &&
        ! empty( $item['variation_id'] ) &&
        (int) $_POST['variation_id'] !== (int) $item['variation_id']
      ){
        continue;
      }
      $remove[] = $key;
    }
  }

  foreach( $remove as $key ){
    $cart->set_quantity( $key, 0 );
  }

  ob_start();
  woocommerce_mini_cart();
  $mini_cart = ob_get_clean();

  $data = array(
    'success' => true,
    'fragments' => apply_filters( 'woocommerce_add_to_cart_fragments', array(
        'div.widget_shopping_cart_content' => '<div class="widget_shopping_cart_content">' . $mini_cart . '</div>',
      )
    ),
    'cart_hash' => apply_filters( 'woocommerce_add_to_cart_hash', WC()->cart->get_cart_for_session() ? md5( json_encode( WC()->cart->get_cart_for_session() ) ) : '', WC()->cart->get_cart_for_session() ),
    'cart_quantity' => WC()->cart->get_cart_contents_count(),
  );
  
	wp_send_json( $data );

}

// in cart
add_shortcode('wcpt_in_cart', 'wcpt_in_cart');
function wcpt_in_cart( $atts ){
  global $product;
  $qty = 0;

  if( ! WC()->cart ){
    return;
  }

  foreach( WC()->cart->get_cart() as $item => $values ){
    if( 
      ( 
        $product->get_type() === 'simple' &&
        $product->get_id() === $values['product_id']
      ) || 
      (
        $product->get_type() === 'variation' &&
        $product->get_id() === $values['variation_id']
      )
    ){
      $qty = $values['quantity'];
      break;

    }
    
    if( 
      $product->get_type() === 'variable' &&
      $product->get_id() === $values['product_id']
    ){
      $qty += $values['quantity'];
    }
  }

  $hide = '';
  if( ! $qty ){
    $hide = 'wcpt-disabled';
  }

  ob_start();
  wcpt_icon('shopping-bag');
  $cart_icon = ob_get_clean();

  return '<span class="wcpt-in-cart '. $hide .'">'. $cart_icon . $qty .'</span>';
}

// override search 'no results'
add_action('woocommerce_before_main_content', 'wcpt_search_override');
function wcpt_search_override(){
  global $wp_query;
  if( wcpt_maybe_override_search() ){
    if( ! $wp_query->post_count ){
      $wp_query->post_count = 1;
    }
  }
}

function wcpt_maybe_override_search(){
  if( ! is_search() ){
    return false;
  }

  return wcpt_search_override_enabled();
}

function wcpt_search_override_enabled(){
  $wcpt_settings = wcpt_get_settings_data();
  
  if( ! $ao_settings = $wcpt_settings['archive_override'] ){
    return false;
  }

  // no value
  if( ! $search = $ao_settings['search'] ){
    return false;
  }

  // custom but no value
  if( 
    $search == 'custom' &&
    empty( $ao_settings['search_custom'] )
  ){
    return false;
  }

  // default but no value
  if( 
    $search == 'default' &&
    empty( $ao_settings['default'] )
  ){
    return false;
  }

  return true;
}

/* 3rd party compatibility */

// WPML
//-- pass params through JS
add_action( 'wp_print_scripts', 'wcpt_permit_wmpl_param_js' );
function wcpt_permit_wmpl_param_js(){
  ?>
  <script>
    if( typeof wcpt_persist_params === 'undefined' ){
      var wcpt_persist_params = [];
    }
    wcpt_persist_params.push('lang');
  </script>
  <?php
}
//-- pass params through PHP
add_filter( 'wcpt_permitted_params', 'wcpt_permit_wmpl_param_php', 100, 1 );
function wcpt_permit_wmpl_param_php( $params ){
	if( ! empty( $_GET['lang'] ) ){
    $params[] = 'lang';
	}
	return $params;
}
//-- remove wpml interference in product cats
add_filter('woocommerce_shortcode_products_query', 'wcpt_remove_wpml_sc_hook', 9, 1);
function wcpt_remove_wpml_sc_hook($query_args){
  if( ! empty( $GLOBALS['sitepress'] ) ){
    wcpt_remove_filters_for_anonymous_class('woocommerce_shortcode_products_query', 'WCML_WC_Shortcode_Product_Category', 'translate_category', 10);
  }
  return $query_args;
}
//-- [wcpt_translate]
add_shortcode('wcpt_translate', 'wcpt_translate');
function wcpt_translate($atts= array()){
  $locale = strtolower( get_locale() );
  $string = '';
  if( ! empty( $atts['default'] ) ){
    $string = $atts['default'];
  }
  if( ! empty( $atts[$locale] ) ){
    $string = $atts[$locale];
  }
  return $string;
}

/**
 * Allow to remove method for an hook when, it's a class method used and class don't have variable, but you know the class name :)
 */
function wcpt_remove_filters_for_anonymous_class( $hook_name = '', $class_name = '', $method_name = '', $priority = 0 ) {
	global $wp_filter;
	// Take only filters on right hook name and priority
	if ( ! isset( $wp_filter[ $hook_name ][ $priority ] ) || ! is_array( $wp_filter[ $hook_name ][ $priority ] ) ) {
		return false;
	}
	// Loop on filters registered
	foreach ( (array) $wp_filter[ $hook_name ][ $priority ] as $unique_id => $filter_array ) {
		// Test if filter is an array ! (always for class/method)
		if ( isset( $filter_array['function'] ) && is_array( $filter_array['function'] ) ) {
			// Test if object is a class, class and method is equal to param !
			if ( is_object( $filter_array['function'][0] ) && get_class( $filter_array['function'][0] ) && get_class( $filter_array['function'][0] ) == $class_name && $filter_array['function'][1] == $method_name ) {
				// Test for WordPress >= 4.7 WP_Hook class (https://make.wordpress.org/core/2016/09/08/wp_hook-next-generation-actions-and-filters/)
				if ( is_a( $wp_filter[ $hook_name ], 'WP_Hook' ) ) {
					unset( $wp_filter[ $hook_name ]->callbacks[ $priority ][ $unique_id ] );
				} else {
					unset( $wp_filter[ $hook_name ][ $priority ][ $unique_id ] );
				}
			}
		}
	}
	return false;
}

// WooCommerce Quick View
// -- shortcode
add_shortcode('wcpt_wc_quick_view', 'wcpt_wc_quick_view');
function wcpt_wc_quick_view($atts= array()){
  ob_start();
  if( class_exists('wcqv_frontend') ){
    global $post;
    $options = get_option('wcqv_options'); 
    ?>
    <span class="woocommerce">
      <a 
        data-product-id="<?php echo $post->ID; ?>"
        class="quick_view button" 
      >
        <span><?php echo $options['button_lable']; ?></span>
      </a>
    </span>
    <?php

  }
  return ob_get_clean();
}
// -- html classes
add_action('wp_enqueue_scripts', 'wcpt_wcqv_script');
function wcpt_wcqv_script(){
  ob_start();
  ?>
    jQuery(function($){
      $('body').on('click', '.wcpt-table .quick_view.button', function(){
        var $body = $('body'),
            has_wc = $('body').hasClass('woocommerce'),
            classes = 'wcpt-wc-quick-view ' + ( has_wc ? '' : ' woocommerce ' );
  
        $body.addClass( classes );
  
        $(document).one('closed', '.remodal', function(){
          $body.removeClass(classes);
        });
      })
    })
  <?php
  wp_add_inline_script( 'jquery', ob_get_clean(), 'after' );

  ob_start();
  ?>
  .wcpt-wc-quick-view .wcqv_prev, 
  .wcpt-wc-quick-view .wcqv_next {
    display: none !important;
  }

  .wcpt .woocommerce .quick_view {
    background: #FFF176;
    color: #444;
  }
  <?php
  wp_add_inline_style( 'wcpt', ob_get_clean(), 'after' );
}

// YITH Request a Quote
// -- disable ajax
add_filter( 'wcpt_shortcode_attributes', 'wcpt_yith_ajax_disable', 100, 1 );
function wcpt_yith_ajax_disable($atts) {
  // -- yith request a quote
  if( function_exists('YITH_YWRAQ_Frontend') ){
    $atts['disable_ajax'] = true;
  }

  return $atts;
}
// -- shortcode
add_shortcode('_yith_quote', 'wcpt_yith_quote');
function wcpt_yith_quote(){
  if( function_exists('YITH_YWRAQ_Frontend') ){
    ob_start();
    YITH_YWRAQ_Frontend()->print_button();
    return ob_get_clean();
  }
}

// Product Add-ons
add_action('wcpt_product_row_html_class', 'wcpt_product_addon_html_class', 100, 2);
function wcpt_product_addon_html_class( $html_class, $product ){

  if(
    class_exists( 'WC_Product_Addons_Helper' ) && 
    method_exists( 'WC_Product_Addons_Helper', 'get_product_addons' )
  ){
    $product_addons = WC_Product_Addons_Helper::get_product_addons( $product->get_id() );
    if ( is_array( $product_addons ) && count( $product_addons ) > 0 ) {
      $html_class .= ' wcpt-product-has-addons ';
    }
  
  }else if( function_exists( 'get_product_addons' ) ){
    $product_addons = get_product_addons( $product->get_id() );
    if ( is_array( $product_addons ) && count( $product_addons ) > 0 ) {
      $html_class .= ' wcpt-product-has-addons ';
    }
  
  }

  return $html_class;
}

// -- for variable products
add_action( 'wcpt_before_loop', 'wcpt_product_addon_compatibility' );
function wcpt_product_addon_compatibility(){
  if( ! empty( $GLOBALS['Product_Addon_Display'] ) ){
    remove_action( 'woocommerce_before_variations_form', array( $GLOBALS['Product_Addon_Display'], 'reposition_display_for_variable_product' ) );    
  }
}

// Advanced Woo Search
$wcpt_permit_aws_influence = false;

if( $wcpt_permit_aws_influence ){
  add_action('init', 'wcpt_aws_do_influence');
}else{
  add_action('init', 'wcpt_aws_remove_influence');  
}

function wcpt_aws_remove_influence(){  
  wcpt_remove_filters_for_anonymous_class( 'posts_request', 'AWS_Search_Page', 'filter_posts_request', 10 );
  wcpt_remove_filters_for_anonymous_class( 'the_posts', 'AWS_Search_Page', 'filter_the_posts', 10 );
  wcpt_remove_filters_for_anonymous_class( 'pre_get_posts', 'AWS_Search_Page', 'action_pre_get_posts', 5 );
  wcpt_remove_filters_for_anonymous_class( 'pre_get_posts', 'AWS_Search_Page', 'pre_get_posts_overwrite', 999 );
  wcpt_remove_filters_for_anonymous_class( 'found_posts_query', 'AWS_Search_Page', 'filter_found_posts_query', 5 );
  wcpt_remove_filters_for_anonymous_class( 'woocommerce_layered_nav_link', 'AWS_Search_Page', 'woocommerce_layered_nav_link' );
  wcpt_remove_filters_for_anonymous_class( 'posts_pre_query', 'AWS_Search_Page', 'posts_pre_query', 10 );
  wcpt_remove_filters_for_anonymous_class( 'body_class', 'AWS_Search_Page', 'body_class', 999 );
}

function wcpt_aws_do_influence(){
  if( ! class_exists('AWS_Search') ){
    return;
  }

  // suppress WCPT search
  add_action( 'wcpt_before_apply_user_filters', 'wcpt_aws_suppress_default_search', 100, 1 );
  // capture search results
  add_filter( 'aws_search_results_products_ids', 'wcpt_aws_search_results_products_ids', 100, 2 );
  // use search results
  add_filter( 'wcpt_query_args', 'wcpt_query_args_aws', 200, 1 );
  // persist params js
  add_action( 'wp_print_scripts', 'wcpt_permit_aws_param_js' );
  // permit param in data-wcpt-query-string
  add_filter( 'wcpt_permitted_params', 'wcpt_permit_aws_param_php', 100, 1 );
  // permit param across AJAX
  add_filter( 'wcpt_permitted_shortcode_attributes', 'wcpt_permitted_shortcode_attributes_aws', 100, 1 );

}

// -- suppress WCPT native search filter
function wcpt_aws_suppress_default_search($data){
  $remove = array();
  foreach( $GLOBALS['wcpt_user_filters'] as $index => &$filter ){
    if( $filter['filter'] === 'search' ){
      $remove[] = $index;
    }
  }

  foreach( $remove as $index ){
    unset( $GLOBALS['wcpt_user_filters'][$index] );
  }
  
}

// -- capture search results
function wcpt_aws_search_results_products_ids( $post_ids, $s ){
  $_SESSION['wcpt_aws'] = $post_ids;
  return $post_ids;
}

// -- use search results
function wcpt_query_args_aws( $args ){
	if(
    ! empty( $_SESSION['wcpt_aws'] ) &&
    ! empty( $_GET['type_aws'] )
  ){

    if( empty( $args['post__in'] ) ){
      $args['post__in'] = $_SESSION['wcpt_aws'];
    }else{
      $args['post__in'] = array_intersect( $_SESSION['wcpt_aws'], $args['post__in'] );
    }

	}
	return $args;
}

// -- persist params js
function wcpt_permit_aws_param_js(){
  ?>
  <script>
    if( typeof wcpt_persist_params === 'undefined' ){
      var wcpt_persist_params = [];
    }
    wcpt_persist_params.push('type_aws', 'post_type', 's');
  </script>
  <?php
}

// -- permit param in data-wcpt-query-string
function wcpt_permit_aws_param_php( $params ){
	if( ! empty( $_GET['type_aws'] ) ){
    $params[] = 'type_aws';
	}
	return $params;
}

// -- permit param across AJAX
function wcpt_permitted_shortcode_attributes_aws($permitted){
  $permitted[] = 'type_aws';
  return $permitted;
}


add_shortcode('wcpt_upsell_skus', 'wcpt_upsell_skus');
function wcpt_upsell_skus(){
  global $product;

  if( $upsells = $product->get_upsell_ids() ){
    $mkp = '';
    $query = new WP_Query(array(
      'post__in' => $upsells,
      'post_type' => 'product'
    ));

    if( $query->have_posts() ){
      while( $query->have_posts() ){
        $query->the_post();
        global $post;
        $product = wc_get_product($post->ID);
        $mkp .= ', '. $product->get_sku();
      }
    }else{
      $mkp .= 'none';
    }

    if( $mkp ){
      return '<span class="wcpt-upsell-skus">'. trim( $mkp, ' ,' ) .'</span>';
    }

  }
}

// main search override
add_action('init', 'wcpt_wc_search_override');
function wcpt_wc_search_override(){
  if( wcpt_search_override_enabled() ){
    add_action( 'pre_get_posts', 'wcpt_wc_search_override__modify_main_query', 1000, 1 );
    add_filter( 'wcpt_query_args', 'wcpt_wc_search_override__use_cached_ids', 100, 1 );
  }  
}

// -- fresh search
function wcpt_wc_search_override__modify_main_query( $query ){
  if(
    ! wp_doing_ajax() &&
    $query->is_main_query() &&
    is_search()
  ){

    // drop main search
    define( 'WCPT_QV_S', get_query_var('s') );
    define( 'WCPT_QV_POSTS_PER_PAGE', get_query_var('posts_per_page') );
    define( 'WCPT_QV_PAGE', get_query_var('page') );

    add_action('wp', 'wcpt_wc_search_override__clean_up');

    set_query_var('s', false);
    set_query_var('posts_per_page', -1);
    set_query_var('page', 1);

    // do search, replace post__in with results
    $keyword = $_REQUEST['s'];

    $settings = wcpt_get_settings_data();

    $search_settings = $settings['search'];
    $ov_settings = $search_settings['override_settings'];

    $target = empty( $ov_settings['target'] ) ? array() : $ov_settings['target'];
    $custom_fields = empty( $ov_settings['custom_fields'] ) ? array() : $ov_settings['custom_fields'];
    $attributes = empty( $ov_settings['attributes'] ) ? array() : $ov_settings['attributes'];

    $keyword_separator = ' ';
    
    $filter_info = array(
      'keyword' => $keyword,
      'target'	=> $target,
      'custom_fields'	=> $custom_fields,
      'attributes' => $attributes,
      'keyword_separator'	=> $keyword_separator,
    );

    
    $post__in = array();
    wcpt_search($filter_info, $post__in);
    set_query_var('post__in', $post__in);

    // cache search result ids
    define( 'WCPT_SEARCH_IDS', $post__in );
    add_action('wp', 'wcpt_wc_search_override__cache_ids');
  }
}

function wcpt_wc_search_override__clean_up(){
  set_query_var('s', WCPT_QV_S);
  set_query_var('post_per_page', WCPT_QV_POSTS_PER_PAGE);
  set_query_var('page', WCPT_QV_PAGE);
}

function wcpt_wc_search_override__cache_ids(){
  $_SESSION['wcpt_search_ids'] = WCPT_SEARCH_IDS;
}

function wcpt_wc_search_override__use_cached_ids( $query_vars ){

  $data = wcpt_get_table_data();
  $sc_attrs = $data['query']['sc_attrs'];

  if( 
    empty( $sc_attrs['_archive'] ) ||
    empty( $sc_attrs['_search'] )
  ){
    return $query_vars;
  }

  $post_ids = isset( $query_vars['post__in'] ) ? $query_vars['post__in'] : false;
  $search_ids = $_SESSION['wcpt_search_ids'];

  if( empty( $post_ids ) ){
    $post_ids = $search_ids;
  }else{
    $post_ids = array_intersect( $search_ids, $post_ids );
  }

  $query_vars['post__in'] = $post_ids;

  return $query_vars;
}

// elementor compatibility
add_shortcode('wcpt_elementor', 'wcpt_elementor');
function wcpt_elementor(){
  ob_start();
  ?>
  <?php if ( have_posts() ) : ?>

    <?php do_action( 'woocommerce_before_shop_loop' ); ?>

    <?php
    if( ! function_exists('wcpt_archive_override') || ! wcpt_archive_override() ){
      woocommerce_product_loop_start();
      if ( wc_get_loop_prop( 'total' ) ) {
        while ( have_posts() ) {
          the_post();
          do_action( 'woocommerce_shop_loop' );
          wc_get_template_part( 'content', 'product' );
        }
      }
      woocommerce_product_loop_end();
    }
    ?>

    <?php do_action( 'woocommerce_after_shop_loop' ); ?>

  <?php else : ?>

    <?php do_action( 'woocommerce_no_products_found' ); ?>

  <?php endif; ?>

  <?php
  return ob_get_clean();
}