<?php 
// update table data based on current version
add_filter( 'wcpt_data', 'wcpt_update_table_data', 10, 1 );
function wcpt_update_table_data( $data ){
  // ensure version number
  // version was not stored in table data before 1.9.0
  if( empty( $data['version'] ) ){
    $data['version'] = '1.8.0';
  }

  // skip update
  // if(
  //   ! empty( $data['version'] ) &&
  //   $data['version'] === WCPT_VERSION 
  // ){
  //   return $data;
  // }

  // backup current data
  update_post_meta( $data['id'], 'wcpt_' . $data['version'], $data );

  // update to 1.9.0
  if( version_compare( $data['version'], '1.9.0', '<' ) ){

    // nav: search filter
    $searches = wcpt_get_nav_elms_ref( 'search', $data );

    foreach( $searches as &$search ){
      $search['attributes'] = array();
      if( $search['custom_fields'] && gettype( $search['custom_fields'] ) === 'string' ){
        $search['custom_fields'] =  array_map( 'trim', preg_split( '/\r\n|\r|\n/', $search['custom_fields'] ) );        
      }

      if( gettype( $search['target'] ) === 'string' ){
        $target = array();

        if( ! empty( $search['target'] ) ){
          foreach( array( 'title', 'content', 'custom_field' ) as $field ){
            if( FALSE !== strrpos( $search['target'], $field ) ){
              $target[] = $field;
            }
          }
  
        }else{
          $target = array( 'title', 'content' );
        }
  
        $search['target'] = $target;
      }

    }

    $data['version'] = '1.9.0';

  }


  // update to 2.0.0
  if( version_compare( $data['version'], '2.0.0', '<' ) ){
    $column_elements = wcpt_get_col_elms_ref( $data, 'attribute' );
    
    // $data['version'] = '2.0.0';

    foreach( $column_elements as &$element ){
      if( ! empty( $element['filter_link_term'] ) ){
        $element['click_action'] = 'trigger_filter';
        unset( $element['filter_link_term'] );
      }
    }

  }

  $data['version'] = WCPT_VERSION;
  
  // update meta
  update_post_meta( $data['id'], 'wcpt', $data );

  return $data;
}

// gets refrence to the requested nav element
// TODO
function wcpt_get_nav_reference( $type, &$arr, &$container = false ){
  if( FALSE === $container ){
    $container = array();
  }

  foreach( $arr as $key => &$val ){
    if(
      $key === 'type' &&
      $val === $type
    ){
      $container[] = $arr;
      break;

    }else if( gettype( $val ) == 'array' ){
      $container = array_merge( $container, wcpt_get_nav_reference( $type, $val, $container ) );

    }
  }

  return $container;
}

// returns references for specific nav filter type 
function wcpt_get_nav_elms_ref( $type= false, &$data = false ){

  if( ! $data ){
    $data = wcpt_get_table_data();
  }

  $navigation =& $data['navigation']['laptop'];
  $rows = array( &$navigation['left_sidebar'][0] );

  foreach( $navigation['header']['rows'] as &$header_row ){
    foreach( $header_row['columns'] as &$column ){
      $rows[] =& $column['template'][0];
    }
  }

  $elements = array();
  foreach( $rows as &$row ){
    if( ! empty( $row['elements'] ) ){
      foreach( $row['elements'] as &$element ){
        if( 
          $type &&
          $type !== $element['type']
        ){
          continue;
        }
  
        $elements[] =& $element;
      }
    }
  }

  return $elements;
}

// returns references for column elements of a type 
function wcpt_get_col_elms_ref( &$data, $type= false ){
  $elements = array();
  foreach( $data['columns'] as &$device ){
    if( empty( $device ) ){ 
      continue;
    }

    foreach( $device as &$column ){
      foreach( $column['cell']['template'] as &$template_row ){
        foreach( $template_row['elements'] as &$element ){
          if( 
            $type &&
            $type !== $element['type']
          ){
            continue;
          }

          $elements[] =& $element;
        }
      }
    }
  }

  return $elements;
}

function wcpt_update_settings_data(){
  $data = json_decode( stripslashes( get_option( 'wcpt_settings', '' ) ), true );

  // ensure version number
  // version was not stored in settings before 1.9.0
  if( empty( $data['version'] ) ){
    $data['version'] = '1.8.0';
  }

  // skip update
  if(
    ! empty( $data['version'] ) &&
    $data['version'] === WCPT_VERSION 
  ){
    return FALSE;
  }

  // backup current data
  update_option( 'wcpt_settings_' . $data['version'], $data );

  // update to 1.9.0
  if( version_compare( $data['version'], '1.9.0', '<' ) ){
    // provide search settings exists
    if( empty( $data['search'] ) ){
      $data['search'] = array(
  
        'stopwords' => implode(", ", array( "i", "me", "my", "myself", "we", "our", "ours", "ourselves", "you", "your", "yours", "yourself", "yourselves", "he", "him", "his", "himself", "she", "her", "hers", "herself", "it", "its", "itself", "they", "them", "their", "theirs", "themselves", "what", "which", "who", "whom", "this", "that", "these", "those", "am", "is", "are", "was", "were", "be", "been", "being", "have", "has", "had", "having", "do", "does", "did", "doing", "a", "an", "the", "and", "but", "if", "or", "because", "as", "until", "while", "of", "at", "by", "for", "with", "about", "against", "between", "into", "through", "during", "before", "after", "above", "below", "to", "from", "up", "down", "in", "out", "on", "off", "over", "under", "again", "further", "then", "once", "here", "there", "when", "where", "why", "how", "all", "any", "both", "each", "few", "more", "most", "other", "some", "such", "no", "nor", "not", "only", "own", "same", "so", "than", "too", "very", "s", "t", "can", "will", "just", "don", "should", "now" )),
        'replacements' => '',
  
        'override_settings' => array(),
  
        'title' => array(
          'enabled' => true,
          'rules' => array(
            'phrase_exact_enabled' => true,
            'phrase_exact_score' => 100,
  
            'phrase_like_enabled' => true,
            'phrase_like_score' => 60,
  
            'keyword_exact_enabled' => true,
            'keyword_exact_score' => 40,
  
            'keyword_like_enabled' => true,
            'keyword_like_score' => 20,
          )
        ),
        'sku' => array(
          'enabled' => true,
          'rules' => array(
            'phrase_exact_enabled' => true,
            'phrase_exact_score' => 100,
  
            'phrase_like_enabled' => true,
            'phrase_like_score' => 60,
  
            'keyword_exact_enabled' => true,
            'keyword_exact_score' => 40,
  
            'keyword_like_enabled' => true,
            'keyword_like_score' => 20,
          )
        ),
        'category' => array(
          'enabled' => true,
          'rules' => array(
            'phrase_exact_enabled' => true,
            'phrase_exact_score' => 100,
  
            'phrase_like_enabled' => true,
            'phrase_like_score' => 60,
  
            'keyword_exact_enabled' => true,
            'keyword_exact_score' => 40,
  
            'keyword_like_enabled' => true,
            'keyword_like_score' => 20,
          )
        ),
        'attribute' => array(
          'enabled' => true,
          'rules' => array(
            'phrase_exact_enabled' => true,
            'phrase_exact_score' => 100,
  
            'phrase_like_enabled' => true,
            'phrase_like_score' => 60,
  
            'keyword_exact_enabled' => true,
            'keyword_exact_score' => 40,
  
            'keyword_like_enabled' => true,
            'keyword_like_score' => 20,
          ),
          'items' => array()
        ),
        'tag' => array(
          'enabled' => true,
          'rules' => array(
            'phrase_exact_enabled' => true,
            'phrase_exact_score' => 100,
  
            'phrase_like_enabled' => true,
            'phrase_like_score' => 60,
  
            'keyword_exact_enabled' => true,
            'keyword_exact_score' => 40,
  
            'keyword_like_enabled' => true,
            'keyword_like_score' => 20,
          )
        ),
        'content' => array(
          'enabled' => true,
          'rules' => array(
            'phrase_exact_enabled' => true,
            'phrase_exact_score' => 100,
  
            'phrase_like_enabled' => true,
            'phrase_like_score' => 60,
  
            'keyword_exact_enabled' => true,
            'keyword_exact_score' => 40,
  
            'keyword_like_enabled' => true,
            'keyword_like_score' => 20,
          )
        ),
        'excerpt' => array(
          'enabled' => true,
          'rules' => array(
            'phrase_exact_enabled' => true,
            'phrase_exact_score' => 100,
  
            'phrase_like_enabled' => true,
            'phrase_like_score' => 60,
  
            'keyword_exact_enabled' => true,
            'keyword_exact_score' => 40,
  
            'keyword_like_enabled' => true,
            'keyword_like_score' => 20,
          )
        ),
        'custom_field' => array(
          'enabled' => true,
          'rules' => array(
            'phrase_exact_enabled' => true,
            'phrase_exact_score' => 100,
  
            'phrase_like_enabled' => true,
            'phrase_like_score' => 60,
  
            'keyword_exact_enabled' => true,
            'keyword_exact_score' => 40,
  
            'keyword_like_enabled' => true,
            'keyword_like_score' => 20,
          ),
          'items' => array()
        ),        
      );
    }

    $data['version'] = '1.9.0';

  }

  // update to 2.0.0
  if( version_compare( $data['version'], '2.0.0', '<' ) ){

    // search override settings
    if( empty( $data['search']['override_settings'] ) ){
      $data['search']['override_settings'] = array(
        'target' => array( 'title', 'content' )
      );
    }

    $data['version'] = '2.0.0';

  }

  $data['version'] = WCPT_VERSION;
  
  // update meta
  update_option( 'wcpt_settings', addslashes( json_encode($data) ) );

  return $data;
}